## Microbial Selection ##
# Two alternative approaches, desscribed in this tutorial: https://malucalle.github.io/Microbiome-Variable-Selection/
# have been implemented in order to detect a microbial signature on pairwise sample comparison.
# See "Supplementary_File_2.xlsx" datasheet for methodological details and results.
# See "Figures" folder for selbal-like plots (Pipeline step 3) for all the pairwise comparison.
# Figure description: in the top part, the "global balance" or "global signature" of the comparison: taxa listed in "A" define the numerator, 
# while those in "B" define the balance denominator. Thus, a significant balance contains a single taxon among those in A group and one within
# the significant ones in the "B"; middle part: boxplot of all balance scores calculated for all possible taxa pairs within the two sample
# groups; bottom part shows the density curves for balance scores in the two considered groups.
### PIPELINE ###
## STEP 1 ##
## Generating a tsv feature-table with taxonomical details.
## INPUT files: "filtered-table.qza", "metadata.tsv", "taxonomy.qza" within ./Micr_Sel/QIIME_input folder.
## Shell commands
conda activate qiime2-2020.8
qiime tools export --input-path filtered-table.qza --output-path .
biom convert -i feature-table.biom -o filtered-feature-table.tsv --to-tsv
qiime tools export --input-path taxonomy.qza --output-path . 
sed '1d' taxonomy.tsv > ref-taxonomy.tsv
biom add-metadata -i feature-table.biom -o filtered-table-with-taxonomy.biom --observation-metadata-fp ref-taxonomy.tsv --observation-header OTUID,taxonomy,confidence --sc-separated taxonomy
biom convert -i filtered-table-with-taxonomy.biom -o filtered-table-with-taxonomy.tsv --to-tsv --header-key taxonomy
#
## STEP 2 ##
## Feature table pre-processing (R environment)
## R commands
# Importing the feature-table, as manipulated by qiime and biom commands. For graphical purposes, artificial identifiers are associated to
# the very long taxonomical classification. "rev_table" is a matrix of 53 rows (samples) and 474 columns (feature counts)
ori_table<-read.table('filtered-table-with-taxonomy.tsv',skip=1,header=TRUE,comment.char='',fill=TRUE,sep='\t')
sample_names<-gsub('X','',as.vector(colnames(ori_table)))
sample_names<-gsub('.','-',as.vector(sample_names),fixed=TRUE)
taxa<-as.vector(ori_table[,'taxonomy'])
taxa_id<-cbind(paste('feature',c(1:dim(ori_table)[1]),sep=''),taxa)
write.table(taxa_id,'taxonomy-ID-association-table.tsv',sep='\t',col.names=FALSE,row.names=FALSE,quote=FALSE)
rev_table<-ori_table[,-1]
rownames(rev_table)<-as.vector(taxa_id[,1])
colnames(rev_table)<-sample_names[-1]
rev_table<-rev_table[,-(dim(rev_table)[2])]
rev_table<-as.data.frame(t(rev_table))
# creating a sample matrix for pairwise group analysis
# "comp_matrix" is a matrix of 53 rows (samples) and 7 columns (number of pairwise comparisons). Entries contain: "1" or "0" for binary class
# comparison; "NO" for samples that are not included in a pairwise comparison.
metadata<-read.table('./QIIME_input/metadata.tsv',sep='\t',header=FALSE,skip=2,fill=TRUE,comment.char='')
response<-as.vector(metadata[,2][match(rownames(rev_table),metadata[,1])])
comp_matrix<-matrix(response,length(response),7)
colnames(comp_matrix)<-c('pouch_paucite','ileum_colon','pouch_colon','pouch_ileum','paucite_ileum','paucite_colon', 'PP_NN')
comp_matrix[,1]<-gsub('Healthy_ileum','NO',comp_matrix[,1])
comp_matrix[,1]<-gsub('Healthy_colon','NO',comp_matrix[,1])
comp_matrix[,1]<-gsub('Paucite',1,comp_matrix[,1])
comp_matrix[,1]<-gsub('Normal_pouch',0,comp_matrix[,1])
comp_matrix[,2]<-gsub('Healthy_ileum',1,comp_matrix[,2])
comp_matrix[,2]<-gsub('Healthy_colon',0,comp_matrix[,2])
comp_matrix[,2]<-gsub('Paucite','NO',comp_matrix[,2])
comp_matrix[,2]<-gsub('Normal_pouch','NO',comp_matrix[,2])
comp_matrix[,3]<-gsub('Healthy_ileum','NO',comp_matrix[,3])
comp_matrix[,3]<-gsub('Healthy_colon',0,comp_matrix[,3])
comp_matrix[,3]<-gsub('Paucite','NO',comp_matrix[,3])
comp_matrix[,3]<-gsub('Normal_pouch',1,comp_matrix[,3])
comp_matrix[,4]<-gsub('Healthy_ileum',0,comp_matrix[,4])
comp_matrix[,4]<-gsub('Healthy_colon','NO',comp_matrix[,4])
comp_matrix[,4]<-gsub('Paucite','NO',comp_matrix[,4])
comp_matrix[,4]<-gsub('Normal_pouch',1,comp_matrix[,4])
comp_matrix[,5]<-gsub('Healthy_ileum',0,comp_matrix[,5])
comp_matrix[,5]<-gsub('Healthy_colon','NO',comp_matrix[,5])
comp_matrix[,5]<-gsub('Paucite',1,comp_matrix[,5])
comp_matrix[,5]<-gsub('Normal_pouch','NO',comp_matrix[,5])
comp_matrix[,6]<-gsub('Healthy_ileum','NO',comp_matrix[,6])
comp_matrix[,6]<-gsub('Healthy_colon',0,comp_matrix[,6])
comp_matrix[,6]<-gsub('Paucite',1,comp_matrix[,6])
comp_matrix[,6]<-gsub('Normal_pouch','NO',comp_matrix[,6])
comp_matrix[,7]<-gsub('Healthy_ileum',0,comp_matrix[,7])
comp_matrix[,7]<-gsub('Healthy_colon',0,comp_matrix[,7])
comp_matrix[,7]<-gsub('Paucite',1,comp_matrix[,7])
comp_matrix[,7]<-gsub('Normal_pouch',1,comp_matrix[,7])
## STEP 3 ##
## Microbial Selection analysis and comparison
# This step partially follows the Variable selection pipeline in : https://uvic-omics.github.io/Microbiome-Variable-Selection/index.html#packages-installation-and-loading
# Download and install the following R Packages: knitr, glmnet, ggplot2, gridExtra, UpSetR, ggforce, grid, selbal, ALDEx2
# From: https://github.com/UVic-omics/Microbiome-Variable-Selection/tree/master/Microbiome_variable_selection_tutorial download  "Functions.R"
# From: https://github.com/UVic-omics/Microbiome-Variable-Selection , download "functions_coda_penalized_regression.R"
# From: https://bioconductor.org/packages/release/bioc/html/ALDEx2.html, download ALDEx2 R Package
source ('functions_coda_penalized_regression.R')
source ('Functions.R')
library(knitr)
library(glmnet)
library(ggplot2)
library(gridExtra)
library(UpSetR)
library(ggforce)
library(grid)
library(selbal)
# An "imputed" rev_table is generated, with the addition of a pseudo-count for taxon with 0 counts.
imp_rev_table<-rev_table
imp_rev_table[imp_rev_table==0]<-1
# Comparison 1: "pouch" (class "1") vs. "paucite" (class "0") samples.
# "coda_lasso" method, 1
sel<-which(comp_matrix[,1]!='NO')
lambdaRange_codalasso(X = imp_rev_table[sel,],as.numeric(as.vector(comp_matrix[sel,1])), lambdaSeq = seq(0.1, 0.5, 0.01))
codalasso_1 <- coda_logistic_lasso(X = imp_rev_table[sel,], y = as.numeric(as.vector(comp_matrix[sel,1])), lambda = 0.45)
codalasso_res1 <- coda_lasso_wrapper(result = codalasso_1, X = imp_rev_table[sel,])
# "clr_lasso" method, 1
sel<-which(comp_matrix[,1]!='NO')
z_1 <- log(imp_rev_table[sel,])
clrx_1 <- apply(z_1, 2, function(x) x - rowMeans(z_1))
glmnet(x = clrx_1, y = as.numeric(as.vector(comp_matrix[sel,1])), family = 'binomial', nlambda = 30)
clrlasso_res1 <- glmnet(x =  clrx_1, y = as.numeric(as.vector(comp_matrix[sel,1])), family = 'binomial', lambda = 0.04745)
clrlasso_res1 <- glmnet_wrapper(clrlasso_res1, X = clrx_1)
# Graphics, 1
Comp_1_clr_pos<-clrlasso_res1$posCoefSelect
Comp_1_clr_neg<-clrlasso_res1$negCoefSelect
selbal_like_plot(pos.names=names(Comp_1_clr_pos), neg.names=names(Comp_1_clr_neg), Y=as.factor(as.vector(comp_matrix[sel,1])),X=imp_rev_table[sel,])
Comp_1_codalasso_pos<-codalasso_res1$posCoefSelect
Comp_1_codalasso_neg<-codalasso_res1$negCoefSelect
selbal_like_plot(pos.names=names(Comp_1_codalasso_pos), neg.names=names(Comp_1_codalasso_neg), Y=as.factor(as.vector(comp_matrix[sel,1])),X=imp_rev_table[sel,])
# Comparison 7: "pouch" + "paucite" (class "1") vs. "healthy_ileum" + "healthy_colon" (class "0") samples
# "coda_lasso", 7
sel<-which(comp_matrix[,7]!='NO')
lambdaRange_codalasso(X = imp_rev_table[sel,],as.numeric(as.vector(comp_matrix[sel,7])), lambdaSeq = seq(0.1, 0.5, 0.01))
codalasso_7 <- coda_logistic_lasso(X = imp_rev_table[sel,], y = as.numeric(as.vector(comp_matrix[sel,7])), lambda = 0.15)
codalasso_res7 <- coda_lasso_wrapper(result = codalasso_7, X = imp_rev_table[sel,])
# "clr_lasso", 7
sel<-which(comp_matrix[,7]!='NO')
z_7 <- log(imp_rev_table[sel,])
clrx_7 <- apply(z_7, 2, function(x) x - rowMeans(z_7))
glmnet(x = clrx_7, y = as.numeric(as.vector(comp_matrix[sel,7])), family = 'binomial', nlambda = 30)
clrlasso_res7 <- glmnet(x =  clrx_7, y = as.numeric(as.vector(comp_matrix[sel,7])), family = 'binomial', lambda = 0.099)
clrlasso_res7 <- glmnet_wrapper(clrlasso_res7, X = clrx_7)
# Graphics, 7
Comp_7_codalasso_pos<-codalasso_res7$posCoefSelect
Comp_7_codalasso_neg<-codalasso_res7$negCoefSelect
selbal_like_plot(pos.names=names(Comp_7_codalasso_pos), neg.names=names(Comp_7_codalasso_neg), Y=as.factor(as.vector(comp_matrix[sel,7])),X=imp_rev_table[sel,])
Comp_7_clr_pos<-clrlasso_res7$posCoefSelect
Comp_7_clr_neg<-clrlasso_res7$negCoefSelect
selbal_like_plot(pos.names=names(Comp_7_clr_pos), neg.names=names(Comp_7_clr_neg), Y=as.factor(as.vector(comp_matrix[sel,7])),X=imp_rev_table[sel,])
# Comparison 2, "healthy_ileum" (class "1") vs. healthy_colon (class "0") samples
# "coda_lasso", 2. No significant results.
sel<-which(comp_matrix[,2]!='NO')
lambdaRange_codalasso(X = imp_rev_table[sel,],as.numeric(as.vector(comp_matrix[sel,2])), lambdaSeq = seq(0.1, 0.5, 0.01))
codalasso_2 <- coda_logistic_lasso(X = imp_rev_table[sel,], y = as.numeric(as.vector(comp_matrix[sel,2])), lambda = 0.15)
codalasso_res2 <- coda_lasso_wrapper(result = codalasso_2, X = imp_rev_table[sel,])
# "clr_lasso", 2
sel<-which(comp_matrix[,2]!='NO')
z_2 <- log(imp_rev_table[sel,])
clrx_2 <- apply(z_2, 2, function(x) x - rowMeans(z_2))
glmnet(x = clrx_2, y = as.numeric(as.vector(comp_matrix[sel,2])), family = 'binomial', nlambda = 30)
clrlasso_res2 <- glmnet(x =  clrx_2, y = as.numeric(as.vector(comp_matrix[sel,2])), family = 'binomial', lambda = 0.0049)
clrlasso_res2 <- glmnet_wrapper(clrlasso_res2, X = clrx_2)
# Graphics, 2
Comp_2_clr_pos<-clrlasso_res2$posCoefSelect
Comp_2_clr_neg<-clrlasso_res2$negCoefSelect
selbal_like_plot(pos.names=names(Comp_2_clr_pos), neg.names=names(Comp_2_clr_neg), Y=as.factor(as.vector(comp_matrix[sel,2])),X=imp_rev_table[sel,])
# Comparison 3, "pouch" (class "1") vs. healthy_colon (class "0") samples
# "coda_lasso", 3. No significant results.
sel<-which(comp_matrix[,3]!='NO')
lambdaRange_codalasso(X = imp_rev_table[sel,],as.numeric(as.vector(comp_matrix[sel,3])), lambdaSeq = seq(0.1, 0.5, 0.01))
codalasso_3 <- coda_logistic_lasso(X = imp_rev_table[sel,], y = as.numeric(as.vector(comp_matrix[sel,3])), lambda = 0.15)
codalasso_res3 <- coda_lasso_wrapper(result = codalasso_3, X = imp_rev_table[sel,])
# "clr_lasso", 3
sel<-which(comp_matrix[,3]!='NO')
z_3 <- log(imp_rev_table[sel,])
clrx_3 <- apply(z_3, 2, function(x) x - rowMeans(z_3))
glmnet(x = clrx_3, y = as.numeric(as.vector(comp_matrix[sel,3])), family = 'binomial', nlambda = 30)
clrlasso_res3 <- glmnet(x =  clrx_3, y = as.numeric(as.vector(comp_matrix[sel,3])), family = 'binomial', lambda = 0.00371)
clrlasso_res3 <- glmnet_wrapper(clrlasso_res3, X = clrx_3)
# Graphics, 3
Comp_3_clr_pos<-clrlasso_res3$posCoefSelect
Comp_3_clr_neg<-clrlasso_res3$negCoefSelect
selbal_like_plot(pos.names=names(Comp_3_clr_pos), neg.names=names(Comp_3_clr_neg), Y=as.factor(as.vector(comp_matrix[sel,3])),X=imp_rev_table[sel,])
# Comparison 4, "pouch" (class "1") vs "healthy_ileum" (class "0")
# "coda_lasso", 4
sel<-which(comp_matrix[,4]!='NO')
lambdaRange_codalasso(X = imp_rev_table[sel,],as.numeric(as.vector(comp_matrix[sel,4])), lambdaSeq = seq(0.1, 0.5, 0.01))
codalasso_4 <- coda_logistic_lasso(X = imp_rev_table[sel,], y = as.numeric(as.vector(comp_matrix[sel,4])), lambda = 0.46)
codalasso_res4 <- coda_lasso_wrapper(result = codalasso_4, X = imp_rev_table[sel,])
# "clr_lasso", 4
sel<-which(comp_matrix[,4]!='NO')
z_4 <- log(imp_rev_table[sel,])
clrx_4 <- apply(z_4, 2, function(x) x - rowMeans(z_4))
glmnet(x = clrx_4, y = as.numeric(as.vector(comp_matrix[sel,4])), family = 'binomial', nlambda = 30)
clrlasso_res4 <- glmnet(x =  clrx_4, y = as.numeric(as.vector(comp_matrix[sel,4])), family = 'binomial', lambda = 0.00365)
clrlasso_res4 <- glmnet_wrapper(clrlasso_res4, X = clrx_4)
# Graphics, 4
Comp_4_codalasso_pos<-codalasso_res4$posCoefSelect
Comp_4_codalasso_neg<-codalasso_res4$negCoefSelect
selbal_like_plot(pos.names=names(Comp_4_codalasso_pos), neg.names=names(Comp_4_codalasso_neg), Y=as.factor(as.vector(comp_matrix[sel,4])),X=imp_rev_table[sel,])
Comp_4_clr_pos<-clrlasso_res4$posCoefSelect
Comp_4_clr_neg<-clrlasso_res4$negCoefSelect
selbal_like_plot(pos.names=names(Comp_4_clr_pos), neg.names=names(Comp_4_clr_neg), Y=as.factor(as.vector(comp_matrix[sel,4])),X=imp_rev_table[sel,])
###
# Comparison 5, "paucite" (class "1") vs. healthy_ileum (class "0")
# "coda_lasso", 5
sel<-which(comp_matrix[,5]!='NO')
lambdaRange_codalasso(X = imp_rev_table[sel,],as.numeric(as.vector(comp_matrix[sel,5])), lambdaSeq = seq(0.1, 1, 0.01))
codalasso_5 <- coda_logistic_lasso(X = imp_rev_table[sel,], y = as.numeric(as.vector(comp_matrix[sel,5])), lambda = 0.74)
codalasso_res5 <- coda_lasso_wrapper(result = codalasso_5, X = imp_rev_table[sel,])
# "clr_lasso", 5
sel<-which(comp_matrix[,5]!='NO')
z_5 <- log(imp_rev_table[sel,])
clrx_5 <- apply(z_5, 2, function(x) x - rowMeans(z_5))
glmnet(x = clrx_5, y = as.numeric(as.vector(comp_matrix[sel,5])), family = 'binomial', nlambda = 30)
clrlasso_res5 <- glmnet(x =  clrx_5, y = as.numeric(as.vector(comp_matrix[sel,5])), family = 'binomial', lambda = 0.00410)
clrlasso_res5 <- glmnet_wrapper(clrlasso_res5, X = clrx_5)
# Graphics, 5
Comp_5_codalasso_pos<-codalasso_res5$posCoefSelect
Comp_5_codalasso_neg<-codalasso_res5$negCoefSelect
selbal_like_plot(pos.names=names(Comp_5_codalasso_pos), neg.names=names(Comp_5_codalasso_neg), Y=as.factor(as.vector(comp_matrix[sel,5])),X=imp_rev_table[sel,])
Comp_5_clr_pos<-clrlasso_res5$posCoefSelect
Comp_5_clr_neg<-clrlasso_res5$negCoefSelect
selbal_like_plot(pos.names=names(Comp_5_clr_pos), neg.names=names(Comp_5_clr_neg), Y=as.factor(as.vector(comp_matrix[sel,5])),X=imp_rev_table[sel,])
###
# Comparison 6, "paucite" (class "1") vs. healthy_colon(class "0")
# "coda_lasso", 6
sel<-which(comp_matrix[,6]!='NO')
lambdaRange_codalasso(X = imp_rev_table[sel,],as.numeric(as.vector(comp_matrix[sel,6])), lambdaSeq = seq(0.1, 1.9, 0.01))
codalasso_6 <- coda_logistic_lasso(X = imp_rev_table[sel,], y = as.numeric(as.vector(comp_matrix[sel,6])), lambda = 1.56)
codalasso_res6 <- coda_lasso_wrapper(result = codalasso_6, X = imp_rev_table[sel,])
# "clr_lasso", 6
sel<-which(comp_matrix[,6]!='NO')
z_6 <- log(imp_rev_table[sel,])
clrx_6 <- apply(z_6, 2, function(x) x - rowMeans(z_6))
glmnet(x = clrx_6, y = as.numeric(as.vector(comp_matrix[sel,6])), family = 'binomial', nlambda = 30)
clrlasso_res6 <- glmnet(x =  clrx_6, y = as.numeric(as.vector(comp_matrix[sel,6])), family = 'binomial', lambda = 0.00485)
clrlasso_res6 <- glmnet_wrapper(clrlasso_res6, X = clrx_6)
# Graphics, 6
Comp_6_codalasso_pos<-codalasso_res6$posCoefSelect
Comp_6_codalasso_neg<-codalasso_res6$negCoefSelect
selbal_like_plot(pos.names=names(Comp_6_codalasso_pos), neg.names=names(Comp_6_codalasso_neg), Y=as.factor(as.vector(comp_matrix[sel,6])),X=imp_rev_table[sel,])
Comp_6_clr_pos<-clrlasso_res6$posCoefSelect
Comp_6_clr_neg<-clrlasso_res6$negCoefSelect
selbal_like_plot(pos.names=names(Comp_6_clr_pos), neg.names=names(Comp_6_clr_neg), Y=as.factor(as.vector(comp_matrix[sel,6])),X=imp_rev_table[sel,])
## STOP ##
# Session Info
# R version 3.6.1 (2019-07-05)
# Platform: x86_64-w64-mingw32/x64 (64-bit)
# Running under: Windows 10 x64 (build 18363)
# Matrix products: default
# locale:
# [1] LC_COLLATE=Italian_Italy.1252  LC_CTYPE=Italian_Italy.1252    LC_MONETARY=Italian_Italy.1252 LC_NUMERIC=C                  
# [5] LC_TIME=Italian_Italy.1252    
# attached base packages:
# [1] grid      stats     graphics  grDevices utils     datasets  methods   base     
# other attached packages:
# [1] selbal_0.1.0  ggforce_0.3.2 UpSetR_1.4.0  gridExtra_2.3 ggplot2_3.3.2 glmnet_4.0-2  knitr_1.30    Matrix_1.2-17
# loaded via a namespace (and not attached):
 # [1] Rcpp_1.0.5        pillar_1.4.7      compiler_3.6.1    plyr_1.8.6        iterators_1.0.13  tools_3.6.1       lifecycle_0.2.0  
 # [8] tibble_3.0.4      gtable_0.3.0      lattice_0.20-38   pkgconfig_2.0.3   rlang_0.4.9       foreach_1.5.1     rstudioapi_0.13  
# [15] xfun_0.19         withr_2.3.0       dplyr_1.0.2       generics_0.1.0    vctrs_0.3.6       tidyselect_1.1.0  glue_1.4.2       
# [22] R6_2.5.0          survival_2.44-1.1 polyclip_1.10-0   farver_2.0.3      tweenr_1.0.1      purrr_0.3.4       magrittr_2.0.1   
# [29] scales_1.1.1      codetools_0.2-16  ellipsis_0.3.1    splines_3.6.1     MASS_7.3-51.4     shape_1.4.5       colorspace_2.0-0 
# [36] munsell_0.5.0     crayon_1.3.4 














